#include <Wire.h>
#include <SPI.h>

#if    defined  _BOARD_OLIMEX_PIC32_PINGUINO_
  #define  UEXT_CS_TRIS  TRISFbits.TRISF0
  #define  UEXT_CS_LAT   LATFbits.LATF0
#elif  defined  _BOARD_OLIMEX_PIC32_PINGUINO_OTG_
  #define  UEXT_CS_TRIS  TRISFbits.TRISF0
  #define  UEXT_CS_LAT   LATFbits.LATF0
#elif  defined  _BOARD_OLIMEX_PIC32_PINGUINO_MICRO_
  #define  UEXT_CS_TRIS  TRISFbits.TRISF0
  #define  UEXT_CS_LAT   LATFbits.LATF0
#elif  defined  _BOARD_OLIMEX_PIC32_T795H_
  #define  UEXT_CS_TRIS  TRISBbits.TRISB11
  #define  UEXT_CS_LAT   LATBbits.LATB11
#elif  defined  _BOARD_OLIMEX_DUINOMITE_MEGA_
  #define  UEXT_EN_TRIS  TRISBbits.TRISB13
  #define  UEXT_EN_LAT   LATBbits.LATB13

  #define  UEXT_CS_TRIS  TRISBbits.TRISB11
  #define  UEXT_CS_LAT   LATBbits.LATB11
#elif  defined  _BOARD_OLIMEX_DUINOMITE_EMEGA_
  #define  UEXT_EN_TRIS  TRISGbits.TRISG13
  #define  UEXT_EN_LAT   LATGbits.LATG13

  #define  UEXT_CS_TRIS  TRISDbits.TRISD14
  #define  UEXT_CS_LAT   LATDbits.LATD14
#endif

enum _Options
{
  INVALID_OPTION = 0,
  PIN_MODE,
  DIGITAL_WRITE,
  DIGITAL_READ,
  ANALOG_READ,
  I2C_SEND,
  I2C_RECEIVE,
  SPI_SEND,
  SPI_RECEIVE,
  SPI_SEND_RECEIVE
};

int Choice;

long int ReadNumber (void)
{
  long int Number=0, Negative_Flag = 1;
  char c;
  while (1)
  {
    c = Serial.read ();
    if (c==-1)
      continue;
    if (c == '-')
    {
      Negative_Flag = -1;
      continue;
    }
    if ((c == 10) || (c == 13))
      break;
    Number = Number * 10 + (c - '0');
  }
  return Number*Negative_Flag;
}

void Wait_to_Press_Any_Key (void)
{
  while (Serial.read () == -1);
}

void Pin_Mode (void)
{
  int Pin, Mode;
  Serial.print ("Input pin number: ");
  Pin = ReadNumber ();
  Serial.println (Pin);
  
  Serial.print ("Input pin mode (0-input; 1-output): ");
  Mode = ReadNumber ();
  Serial.println (Mode);
  
  pinMode (Pin, Mode);
  Serial.println ("Press any key to continue!");
  Wait_to_Press_Any_Key ();
}

void Digital_Write (void)
{
  int Pin, Level;
  Serial.print ("Input pin number: ");
  Pin = ReadNumber ();
  Serial.println (Pin);
  
  Serial.print ("Input pin level: ");
  Level = ReadNumber ();
  Serial.println (Level);
  
  digitalWrite (Pin, Level);
  Serial.println ("Press any key to continue!");
  Wait_to_Press_Any_Key ();
}

void Digital_Read (void)
{
  int Pin, Level;
  Serial.print ("Input pin number: ");
  Pin = ReadNumber ();
  Serial.println (Pin);

  Level = digitalRead (Pin);
  Serial.print ("Pin level is: ");
  Serial.println (Level);
  Serial.println ("Press any key to continue!");
  Wait_to_Press_Any_Key ();
}

void Analog_Read (void)
{
  int Pin, Level;
  Serial.print ("Input pin number: ");
  Pin = ReadNumber ();
  Serial.println (Pin);

  Level = analogRead (Pin);
  Serial.print ("Pin level is: ");
  Serial.println (Level);
  Serial.println ("Press any key to continue!");
  Wait_to_Press_Any_Key ();
}

void I2C_Send (void)
{
  unsigned char Address, Data[10];
  int Number, i;
  Serial.print ("Slave address: ");
  Address = ReadNumber ();
  Serial.println ((int)Address);

  Serial.print ("Number of data bytes: ");
  Number = ReadNumber ();
  Serial.println (Number);

  for (i=0; i<Number; i++)
  {
    Serial.print ("Input data: ");
    Data[i] = ReadNumber ();
    Serial.println ((int)Data[i]);
  }

  Wire.beginTransmission (Address);
  Wire.send (Data, Number);
  Wire.endTransmission ();
  Serial.println ("Press any key to continue!");
  Wait_to_Press_Any_Key ();
}

void I2C_Receive (void)
{
  unsigned char Address, In_Data[10], Out_Data[10];
  int In_Number, Out_Number, i;
  Serial.print ("Slave address: ");
  Address = ReadNumber ();
  Serial.println ((int)Address);

  Serial.print ("Number of input data bytes: ");
  In_Number = ReadNumber ();
  Serial.println (In_Number);

  Serial.print ("Number of output data bytes: ");
  Out_Number = ReadNumber ();
  Serial.println (Out_Number);

  for (i=0; i<In_Number; i++)
  {
    Serial.print ("Input data: ");
    In_Data[i] = ReadNumber ();
    Serial.println ((int)In_Data[i]);
  }
  
  while (Serial.read() != 0x0A)
  {
    delay (500);
    Wire.beginTransmission (Address);
    Wire.send (In_Data, In_Number);
    Wire.endTransmission ();
  
  
    Wire.requestFrom(Address, (unsigned char) Out_Number);
  
    i=0;
    while(Wire.available())    // slave may send less than requested
    {
      Out_Data[i++] = Wire.receive(); // receive a byte as character
    }
  
    for (i=0; i<Out_Number; i++)
    {
      Serial.print ("Output Data: ");
      Serial.println ((int)Out_Data[i]);
    }
  }

  Serial.println ("Press any key to continue!");
  Wait_to_Press_Any_Key ();
}

void SPI_Send (void)
{
  unsigned char Data;
  int Number, i;

  Serial.print ("Number of data bytes which to be sent: ");
  Number = ReadNumber ();
  Serial.println ((int)Number);

  UEXT_CS_LAT = 0;
  delay (10);
  for (i=0; i<Number; i++)
  {
    Serial.print ("Input data: ");
    Data = ReadNumber ();
    Serial.println ((int)Data);
    SPI.transfer (Data);
  }
  UEXT_CS_LAT = 1;

  Serial.println ("Press any key to continue!");
  Wait_to_Press_Any_Key ();
}

void SPI_Receive (void)
{
  unsigned char Data[10];
  int Number, i;

  Serial.print ("Number of data bytes which to be received: ");
  Number = ReadNumber ();
  Serial.println ((int)Number);

  UEXT_CS_LAT = 0;
  delay (10);
  for (i=0; i<Number; i++)
  {
    Data[i] = SPI.transfer (0x55);
  }
  UEXT_CS_LAT = 1;

  for (i=0; i<Number; i++)
  {
    Serial.print ("Received Data: ");
    Serial.println ((int)Data[i]);
  }

  Serial.println ("Press any key to continue!");
  Wait_to_Press_Any_Key ();
}

void SPI_Send_Receive (void)
{
  unsigned char Send, Receive;
  
  Serial.print ("Input data: ");
  Send = ReadNumber ();
  Serial.println ((int)Send);

  UEXT_CS_LAT = 0;
  delay (10);
  Receive = SPI.transfer (Send);
  UEXT_CS_LAT = 1;

  Serial.print ("Received data: ");
  Serial.println ((int)Receive);
}

void setup ()
{
  Serial.begin (9600);  // UART Init

  Wire.begin ();  // I2C Init

  SPI.begin();    // SPI Init
  
  #if (defined _BOARD_OLIMEX_DUINOMITE_MEGA_) || (defined _BOARD_OLIMEX_DUINOMITE_EMEGA_)
  UEXT_EN_TRIS = 0;
  UEXT_EN_LAT = 0;
  #endif
  UEXT_CS_TRIS = 0;

  Serial.println ("Press any key to continue!");
  Wait_to_Press_Any_Key ();
}

void loop ()
{
  Serial.println ("\n\rMenu:");
  Serial.println ("1) Pin Mode");
  Serial.println ("2) Digital Write");
  Serial.println ("3) Digital Read");
  Serial.println ("4) Analog Read");
  Serial.println ("5) I2C Send");
  Serial.println ("6) I2C Receive");
  Serial.println ("7) SPI Send");
  Serial.println ("8) SPI Receive");
  Serial.println ("9) SPI Send and Receive");

  Serial.print ("Input your choice: ");
  Choice = ReadNumber ();
  Serial.println (Choice);
  
  switch (Choice)
  {
    case PIN_MODE:
      Pin_Mode ();
    break;
    case DIGITAL_WRITE:
      Digital_Write ();
    break;
    case DIGITAL_READ:
      Digital_Read ();
    break;
    case ANALOG_READ:
      Analog_Read ();
    break;
    case I2C_SEND:
      I2C_Send ();
    break;
    case I2C_RECEIVE:
      I2C_Receive ();
    break;
    case SPI_SEND:
      SPI_Send ();
    break;
    case SPI_RECEIVE:
      SPI_Receive ();
    break;
    case SPI_SEND_RECEIVE:
      SPI_Send_Receive ();
    break;
    default:
      Serial.println ("Invalid input!");
    break;
  }
}
